% Author: Ibrahim Ghafir
% Datasets: https://doi.org/10.17028/rd.lboro.4746844.v1
% Licence:CCBY 4.0 

%  1- load the data and assign x to the test metrics.
load deauth_data.txt

% TTL value doesn't exist in management frames, so there is no column for
% TTL in deauth_data.txt, so we put zeros for TTL column in x data (to 
% have six metrics as the other datasets).

x(:,1:2) = deauth_data (:,1:2);
x(:,3) = zeros;
x(:,4:6) = deauth_data (:,3:5);

%x1 = mitm_attack02_data(:,1); rssi = x1;
%x2 = mitm_attack02_data(:,2); injection_rate = x2;
%x3 = mitm_attack02_data(:,3); nav = x3;
%x4 = mitm_attack02_data(:,4); ttl = x4;
%x5 = mitm_attack02_data(:,5); sequence_number = x5;
%x6 = mitm_attack02_data(:,6); inter_arrival_time = x6;

%% 2- label the data, based on NAV = 314

for i=1:size(x,1)  % size(x,1) is the number of rows in the matrix x
    if (x(i,4)==314)
        x(i,7)=1;
    else
         x(i,7)=0;
    end
end

%% change the TTL values (from 255 to 50) and change change the sequence 
%  number, note that we don't change the NAV value for evaluation

% change TTL value

%for i=1:size(x,1)  % size(x,1) is the number of rows in the matrix x
%    if (x(i,3)==255)
%        x(i,3)=50;
%    end
%end

% change sequence number value

for i=size(x,1):-1:2  % size(x,1) is the number of rows in the matrix x
    x(i,5)=x(i,5)-x(i-1,5);
end

x(1,5)=0;

%% partitioning the data into:
%  profile_data: used to build the profiles of normality and abnormality
%                beliefs (80% of the data). 
%  test_data: used to assign the beliefs and test Dumster Shafer theory
%             (20% of the data).

%test_data=x(1:end*0.3,:);
%profile_data=[x(1:69,:); x(128:203,:)];  % N~114 samples , A~28 samples

% Deauth data samples (x data):
% 1-39     normal samples,  40
% 40-44    attack           5
% 45-46    normal           2
% 47-75    attack           29
% 76       normal           1
% 77-95    attack           19
% 96       normal           1
% 97-116   attack           20
% 117      normal           1
% 118-127  attack           110
% 128-203  normal           76

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%profile_data=x(end*0.3+1:end,:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%test_data=x(1:end*0.3,:);

profile_data=x(end*0.311+1:end,:);
test_data=x(1:end*0.311,:);

%%%%%%profile_data=x(118:203,:);
%test_data=x(1:44,:);
%%%%%test_data=x(1:117,:);

%profile_data=x(128:203,:);   % this is only normal samples
%test_data=x(1:44,:);
%test_data=x(1:127,:);

%test_data=x; % This is to evaluate the algorithms on all data (test data 100%)
%test_data=x(111:184,:);  % N~56 samples , A~14 samples
%test_data=x(70:127,:); % this is the rest of data
%test_data=x; % This is to evaluate the algorithms on all data (test data 100%)
%% Normality belief assignment (gaussian distribution)

% kmeans clustering

%  apply kmeans clustering based on all metrics of the frame, to filter the
%  significant outliers, and to get more accurate normality profile

kmeans_data = profile_data(:,[1 2 4 5 6]);

% this is to plot the data (only 2 dimensions)
%figure(1);
%plot(kmeans_data(:,1),kmeans_data(:,2),'.');

opts = statset('Display','final');
[idx,C] = kmeans(kmeans_data,2,'Distance','cityblock',...
    'Replicates',5,'Options',opts);

%opts = statset('Display','final');
%[idx,C] = kmeans(kmeans_data,2);

% this is to plot the clusered data in different coulours, besides the
% centroids

%figure(2);
%plot(kmeans_data(idx==1,1),kmeans_data(idx==1,2),'r.','MarkerSize',12)
%hold on
%plot(kmeans_data(idx==2,1),kmeans_data(idx==2,2),'b.','MarkerSize',12)
%plot(C(:,1),C(:,2),'kx',...
%     'MarkerSize',15,'LineWidth',3)
%legend('Cluster 1','Cluster 2','Centroids',...
%       'Location','NW')
%title 'Cluster Assignments and Centroids'
%hold off

% to see the number of elements of each cluster
%figure(1);
%histogram (idx);

% label the data based on kmeans clustring, considering the larger cluster
% is the normal one, and the smaller cluster is the outliers.

% kmeans define each cluster with a number, here we have 2 clusters, so
% each alement in the data is clustered into '1' or '2'.

% to find the most frequent value in idx (the larger cluster) 

normal_cluster = mode(idx);

for i=1:size(profile_data,1)  % size(x,1) is the number of rows in the matrix x
    if (idx(i,1)==normal_cluster)
        profile_data(i,8)=0;
    else
         profile_data(i,8)=1;
    end
end 

cleaner_data=profile_data;

outliers_rows = find(cleaner_data(:,8)==1);
cleaner_data(outliers_rows,:) = [];

% Gaussian distribution and normality belief assignment
% we build the normality profile based on the cleaner_data (calculate u and
% sigma) and then used the previous values to apply gaussian formula on the
% test_dada to find the normality belief.

% for now, lets consider al the metrics are gaussian distributed, however,
% we might need to consider the bernoli distribution for 2 and 4 metrics.

%%% Different distributions are to be utilised for normality belief assignment:

%Gaussian distribution: for RSSI, seq
%Categorical distribution (Bernoulli): for Rate, NAV
%Exponential distribution: for IAT
%%%%%%%%

% for Gaussian distribution

for metric = [1 5 6];

gaussian_data = [];
normality_belief = [];

gaussian_data = cleaner_data(:,metric);

normality_belief = (normpdf(test_data(:,metric),mean(gaussian_data),std(gaussian_data)) ...
    / normpdf(mean(gaussian_data),mean(gaussian_data),std(gaussian_data)))*100;

test_data(:,metric+10) = normality_belief;

end

% for Categorical distribution

for metric = [2 4]

categorical_data = [];
normality_belief = [];
categorical_data = cleaner_data(:,metric);
categorical_matrix = [];

categorical_unique = unique(categorical_data);

for i = 1:length(categorical_unique)
occurrences(i) = sum(categorical_data==categorical_unique(i));
frequency(i) = (occurrences(i) / length(categorical_data))*100;
% the folowing matrix gives the normlity belief ofeach unique value
categorical_matrix = [categorical_matrix; categorical_unique(i), frequency(i)];
end

for i=1:size(test_data,1)
    y=ismember(test_data(i,metric),categorical_matrix(:,1));
        if y==0
            normality_belief(i) = 0;
        else
            for j=1:size(categorical_matrix,1)
                if test_data(i,metric) == categorical_matrix(j,1)
                    normality_belief(i) = categorical_matrix(j,2);
                end
            end
        end
end

test_data(:,metric+10) = normality_belief;

end

% for Exponential distribution
%{
% the follwoing 3 lines are for debugging purpose
exp_metric_mu = [];
exp_metric_min = [];
exp_metric_P_max = [];

for metric = 6;

exponential_data = [];
normality_belief = [];

exponential_data = cleaner_data(:,metric);
exponential_mean = mean(exponential_data);

exp_metric_mu(metric) = exponential_mean; % for debugging purpose
metric_min = min(exponential_data);
P_max = exppdf(metric_min,exponential_mean);

 for i=1:size(test_data,1)
     if test_data(i,metric) < metric_min
        metric_min = test_data(i,metric);
        P_max = exppdf(metric_min,exponential_mean);
     end
            
   normality_belief(i) = (exppdf(test_data(i,metric),exponential_mean) / P_max)*100;

 end
 
 exp_metric_min(metric) = metric_min;
 exp_metric_P_max(metric) = P_max;
 test_data(:,metric+10) = normality_belief;

end
%}
%% Abnormality belief assignment (Local Reachability Density)

% For each metric in the profile_data, we calculate the lrd for each
% sample, and find lrd_max. This includes using the k-nearest algorithm

% for each metric in the test_data, we calculate the lrd based on the
% profile_data. This includes using the k-nearest algorithm

% To assign the abnormality belief, we scale the calculated lrd to lrd_max.
% lrd_max is used as a reference.

% 1- find the k-nearest neighbours of each sample (frame) in the 
% profile_data

% keep only the unique values in the knn data, to avoid the zero distances
% resulting from the dublicator values.


% think of calculation the local reachability densit of all samples,
% then chose the max as the 100% normal
% then calulate the local reaxhability desity of your test sample
% then scale to the max lrd
% abnornality beief = [1-(lrd/lrd_max)]*100


% assign k, based on the number of unique values

for metric=[1 2 4 5 6]

knn_data=[];

knn_data=unique(profile_data(:,metric));

if size(knn_data,1)<21
    k=round(0.7*size(knn_data,1)); 
else
    k=20;
end

for i=1:size(knn_data,1) % size(x,1) is the number of rows in the matrix x
    knn_data_i=[];
    neigbours_p=[];
    distances_p=[];
   % k_dist_p=[];
    
    knn_data_i=knn_data;
    p=knn_data_i(i);
    knn_data_i(i)=[]; % to avoid the zero distance between identical samples
   
    [neigbours_p,distances_p]=knnsearch(knn_data_i, p,'k',k);
    %k_dist_p=distances_p(k); % not required for calculating lrd_k_p
    
    k_dist_o=[];
    dist_p_o=[];
    reach_dist_p_o=[];
    
    for j=1:size(neigbours_p,2)
        knn_data_i_j=[];
        neigbours_o=[];
        distances_o=[];
        
        knn_data_i_j=knn_data_i;
        o=knn_data_i_j(neigbours_p(j));
        knn_data_i_j(neigbours_p(j))=[]; % delete the value of o, to avoid the duplicator samples
        knn_data_i_j=[knn_data_i_j;p]; % return the value of p previously deleted
           % and add it t the end of the matrix so you don't affect the
           % indexes of neigbours_p when they are used in the next for loop.
    
        [neigbours_o,distances_o]=knnsearch(knn_data_i_j, o,'k',k);
        k_dist_o(j)=distances_o(k);
        dist_p_o(j)=distances_p(j);
        %neighbours(j,1:k)=neigbours_o; % not required for calculating lrd_k_p
        reach_dist_p_o(j)=max(k_dist_o(j),dist_p_o(j));

    end
    
    lrd_p(i)=1/(sum(reach_dist_p_o)/k); % lrd for the profile_data sample

end

lrd_max(metric)= max(lrd_p);

lrd_min(metric)= min(lrd_p);

end

% for the test_data

for metric=[1 2 4 5 6]

knn_data=[];

knn_data=unique(profile_data(:,metric));

if size(knn_data,1)<21
    k=round(0.7*size(knn_data,1)); 
else
    k=20;
end

for i=1:size(test_data,1) % size(x,1) is the number of rows in the matrix x
    knn_data_i=[];
    neigbours_p=[];
    distances_p=[];
   % k_dist_p=[];
    
    knn_data_i=knn_data;
    p=test_data(i, metric);
    
    if ismember(p, knn_data_i)==1
        p_exist=1;
        knn_data_i(find(knn_data_i==p))=[]; % to delete if there is a duplicate value
    else
        p_exist=0;
    end
   
    [neigbours_p,distances_p]=knnsearch(knn_data_i, p,'k',k);
    %k_dist_p=distances_p(k); % not required for calculating lrd_k_p
    
    k_dist_o=[];
    dist_p_o=[];
    reach_dist_p_o=[];
    
    for j=1:size(neigbours_p,2)
        knn_data_i_j=[];
        neigbours_o=[];
        distances_o=[];
        
        knn_data_i_j=knn_data_i;
        o=knn_data_i_j(neigbours_p(j));
        knn_data_i_j(neigbours_p(j))=[]; % delete the value of o, to avoid the duplicator samples
        
        if p_exist==1
            knn_data_i_j=[knn_data_i_j;p]; % return the value of p previously deleted
        end
    
        [neigbours_o,distances_o]=knnsearch(knn_data_i_j, o,'k',k);
        k_dist_o(j)=distances_o(k);
        dist_p_o(j)=distances_p(j);
        %neighbours(j,1:k)=neigbours_o; % not required for calculating lrd_k_p
        reach_dist_p_o(j)=max(k_dist_o(j),dist_p_o(j));

    end
    
    lrd_p_t(i)=1/(sum(reach_dist_p_o)/k);  % lrd for the test_data sample
    
    if lrd_p_t(i) < lrd_min(metric)  % update lrd_min
        lrd_min(metric)=lrd_p_t(i);
    end
    
    if lrd_p_t(i) > lrd_max(metric)  % update lrd_max
        lrd_max(metric)=lrd_p_t(i);
    end
    
  %  abnormality_belief_1(i)=(lrd_p_t(i)*100)/lrd_min(metric);
    
  %  abnormality_belief_2(i)=1/((lrd_p_t(i)*100)/lrd_min(metric));
    
    abnormality_belief_3(i)=100-((lrd_p_t(i)*100)/lrd_max(metric));
    
    
    
   % test_data(i,metric+20) = abnormality_belief_1(i);
    
  %  test_data(i,metric+30) = abnormality_belief_2(i);
    
    test_data(i,metric+40) = abnormality_belief_3(i);

end

end

%% label the test_data based on Normality and abnormality beliefs

% lrd_max test

label=7;

for metric=[1 2 4 5 6]

for i=1:size(test_data,1)
   if test_data(i,metric+10) > test_data(i,metric+40)
       test_data(i, metric+50) = 0;
   else 
       test_data(i, metric+50) = 1;
   end
end

% calculate P, N, TP, TN, FP, FN, TPR, FPR

P=0; N=0; TP=0; TN=0; FP=0; FN=0;

for i=1:size(test_data,1)
    if test_data(i,label)==1
        P=P+1;
    else N=N+1;
    end
end

for i=1:size(test_data,1)
    if test_data(i,metric+50)==1
        if test_data(i,label)==1
            TP=TP+1;
        else FP=FP+1;
    end
    end
end

for i=1:size(test_data,1)
    if test_data(i,metric+50)==0
        if test_data(i,label)==0
            TN=TN+1;
        else FN=FN+1;
        end
    end
end

TPR = (TP/P)*100;
FPR = (FP/N)*100;
OSR = ((TP+TN)/(TP+FP+TN+FN))*100;
Precision = (TP/(TP+FP))*100;
F_score = 2*(((Precision/100)*(TPR/100))/((Precision/100)+(TPR/100)));

%disp (P); disp (N); disp (TP); disp (TN); disp (FP); disp (FN); disp (TPR); disp (FPR);

% write the results intp (beliefs_results) matrix

beliefs_results(metric,1)=metric; beliefs_results(metric,2)=P; beliefs_results(metric,3)=N;
beliefs_results(metric,4)=TP; beliefs_results(metric,5)=TN; beliefs_results(metric,6)=FP;
beliefs_results(metric,7)=FN; beliefs_results(metric,8)=TPR; beliefs_results(metric,9)=FPR;
beliefs_results(metric,10)=OSR; beliefs_results(metric,11)=Precision; beliefs_results(metric,12)=F_score;

end

%%

% lrd_min test  % this test doesn't give good results, could be deleted

%{

 label=7;

for metric=[1 2 4 5 6]

for i=1:size(test_data,1)
   if test_data(i,metric+10) > test_data(i,metric+30)
       test_data(i, metric+60) = 0;
   else 
       test_data(i, metric+60) = 1;
   end
end

% calculate P, N, TP, TN, FP, FN, TPR, FPR

P=0; N=0; TP=0; TN=0; FP=0; FN=0;

for i=1:size(test_data,1)
    if test_data(i,label)==1
        P=P+1;
    else N=N+1;
    end
end

for i=1:size(test_data,1)
    if test_data(i,metric+60)==1
        if test_data(i,label)==1
            TP=TP+1;
        else FP=FP+1;
    end
    end
end

for i=1:size(test_data,1)
    if test_data(i,metric+60)==0
        if test_data(i,label)==0
            TN=TN+1;
        else FN=FN+1;
        end
    end
end

TPR = (TP/P)*100;
FPR = (FP/N)*100;

%disp (P); disp (N); disp (TP); disp (TN); disp (FP); disp (FN); disp (TPR); disp (FPR);

% write the results intp (iqr_results) matrix

beliefs_results_2(metric,1)=metric; beliefs_results_2(metric,2)=P; beliefs_results_2(metric,3)=N;
beliefs_results_2(metric,4)=TP; beliefs_results_2(metric,5)=TN; beliefs_results_2(metric,6)=FP;
beliefs_results_2(metric,7)=FN; beliefs_results_2(metric,8)=TPR; beliefs_results_2(metric,9)=FPR;

end 
%}


%%%%%% this is to find DS beliefs after findings for BPA for each metric

%% This script is to calculate the uncertainty and adjusted beliefs to be 
% able to apply the Dempster-Shafer theory
% this script writes the adjusted results into a csv file

% before you start
% 1- clear the workplace
% 2- remember that for deauth data, we don't have the TTL value (metric 3)

%load deauth_test_data.mat
%load deauth_test_data_all.mat %(all gauussian)
%%%%%%%load deauth_test_data_multi_distributions.mat %(all gauussian except Rate and NAV (benoulli), and IAT(exponential))
%load deauth_test_data_multi_distributions_normal_traffic_training.mat %(all gauussian except Rate and NAV (benoulli), and IAT(exponential), normal traffic profile data)

%load deauth_test_data_multi_distributions_2.mat %(all gauussian except Rate and NAV (benoulli))

% the test_data coloumns are as follows:
% each row is for one sample, for the coloumns:
% 1-6 metrics values
% 7 ground truth (1 attack, 0 normal)
% 8-10 empty
% 11-16 Normality belief
% 17-40 empty
% 41-46 Abnormality belief
% 47-50 empty
% 51-56 labeling the frames based on each single-metric beliefs 
%      (which one is greater)

%%%%%%%%%%%%%%%%%%%%%%%

% This is to devide the normality and attack belief by 100, as they were
% multiblied by 100 in the original script.

for i=1:size(test_data,1)
    
    for j=[11:16 41:46]
            test_data(i,j) = test_data(i,j)/100;
    end
end

%%%%%%%%%%%%%%%
% This is to change the very small beliefs (which are smaller than 0.01) 
% to the value of 0.01 (for Normality and Attack)


for i=1:size(test_data,1)
    
    for j=[11:16 41:46]
        if test_data(i,j) < 0.01
            test_data(i,j) = 0.01;
        end
    end
end

%%%%%%%%%%%%%%%%%%%%

% this to calculate the uncertainty belief (by dividing the beliefs 
% (smaller/greater)
% write the certainty beliefs into coloumns 61-66

for metric=[1 2 4 5 6]
    
    for i=1:size(test_data,1)
        
        if test_data(i,metric+10) < test_data(i,metric+40)
            test_data(i,metric+60) = test_data(i,metric+10) / test_data(i,metric+40);
        else 
            test_data(i,metric+60) = test_data(i,metric+40) / test_data(i,metric+10);
        end
        
    end
   
end

% this is to adjust the beliefs (to make the sum equal to 1) (normalization)
% write the adjusted beliefs into coloumns ad_n(71-76), ad_a(81-86), ad_u(91-96) 

for metric=[1 2 4 5 6]
    
    for i=1:size(test_data,1)
      
        test_data(i,metric+70) = (test_data(i,metric+10) / ... 
            (test_data(i,metric+10) + test_data(i,metric+40) + test_data(i,metric+60)));        
        
        test_data(i,metric+80) = (test_data(i,metric+40) / ... 
            (test_data(i,metric+10) + test_data(i,metric+40) + test_data(i,metric+60)));
        
        test_data(i,metric+90) = (test_data(i,metric+60) / ... 
            (test_data(i,metric+10) + test_data(i,metric+40) + test_data(i,metric+60)));        
         
    end
   
end

% This is to write the previous calculated beliefs into more organised data
% (adjusted_beliefs)for Dempester-Shafer theory application (write only 
% the coloumns required for DempsterShafer)

% each row is for one sample, for the coloumns:
% 1-6 metrics values
% 7 ground truth (1 attack, 0 normal)
% 8-10 empty
% 11-16 adjusted abnormality belief (attck)
% 17-20 empty
% 21-26 adjusted normality belief (normal)
% 27-30 empty
% 31-36 adjusted uncertainty belief

adjusted_beliefs(:,1:7) = test_data(:,1:7);
adjusted_beliefs(:,11:16) = test_data(:,81:86);
adjusted_beliefs(:,21:26) = test_data(:,71:76);
adjusted_beliefs(:,31:36) = test_data(:,91:96);

% This is to export the adjusted beliefs data into csv file (should be
% written into the same directory)

%csvwrite('adjusted_beliefs_deauth.csv',adjusted_beliefs);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%test = adjusted_beliefs(:,11:36);     % this is to find the minumm non zero belief value
%minimum_belief = min(test(test > 0));

% the minimum belief we get it  6.195889508648017e-11, so we change the
% zero beliefs into 5.195889508648017e-11


% calculate the beliefs of Dempste-Shafer

% apply Dempster-Shafer on all six metrics.

ad = adjusted_beliefs;

for i=1:size(ad,1)
    
    % This for loop is to apply the Dempster-Shafer theory on all six metrics
    % the output of the first two metrics fusion is fused with the third
    % metric and so on.
    
    % Please note that there is no TTL value in this Deauth data
    
    % dempster-shafer for RSSI and Injection rate
    % dempster shafer beliefs are written into coloumns 41,42,43 (attack, normal, uncertainty)
    % the frame label is written into coloumn 44
    
    [ad(i,41),ad(i,42),ad(i,43),ad(i,44)] = ds(ad(i,11),ad(i,21),ad(i,31),ad(i,12),ad(i,22),ad(i,32));
    
    % dempster-shafer for RSSI,Injection rate and TTL
    % dempster shafer beliefs are written into coloumns 46,47,48 (attack, normal, uncertainty)
    % the frame label is written into coloumn 49    
    
    %[ad(i,46),ad(i,47),ad(i,48),ad(i,49)] = ds(ad(i,41),ad(i,42),ad(i,43),ad(i,13),ad(i,23),ad(i,33));
    
    % dempster-shafer for RSSI,Injection rate and NAV
    % dempster shafer beliefs are written into coloumns 51,52,53 (attack, normal, uncertainty)
    % the frame label is written into coloumn 54 
    
    % amend the next line to fit with Deauth data (delete TTL)
    
    %[ad(i,51),ad(i,52),ad(i,53),ad(i,54)] = ds(ad(i,46),ad(i,47),ad(i,48),ad(i,14),ad(i,24),ad(i,34));    
   
    [ad(i,51),ad(i,52),ad(i,53),ad(i,54)] = ds(ad(i,41),ad(i,42),ad(i,43),ad(i,14),ad(i,24),ad(i,34));    
    
    % dempster-shafer for RSSI,Injection rate,NAV and sequence number
    % dempster shafer beliefs are written into coloumns 56,57,58 (attack, normal, uncertainty)
    % the frame label is written into coloumn 59    
    
    [ad(i,56),ad(i,57),ad(i,58),ad(i,59)] = ds(ad(i,51),ad(i,52),ad(i,53),ad(i,15),ad(i,25),ad(i,35));   
        
    % dempster-shafer for RSSI,Injection rate,NAV,sequence number and Inter arrival time (all metrics)
    % dempster shafer beliefs are written into coloumns 61,62,63 (attack, normal, uncertainty)
    % the frame label is written into coloumn 64    
    
    [ad(i,61),ad(i,62),ad(i,63),ad(i,64)] = ds(ad(i,56),ad(i,57),ad(i,58),ad(i,16),ad(i,26),ad(i,36));    
     
end

% This is to find the results of Dempster-Shafer on all metrics, 
% i.e. calculate the TPR and FPR of each fusion step.

% calculate P, N, TP, TN, FP, FN, TPR, FPR

label=7;     % ground truth

ds_all_metrics_results = [];  % to write the results into

for ds_l = [44 49 59 64]   %dempster-shafer label % 45 was deleted for deauth data
    
    P=0; N=0; TP=0; TN=0; FP=0; FN=0;

for i=1:size(ad,1)
    if ad(i,label)==1
        P=P+1;
    else N=N+1;
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==1
        if ad(i,label)==1
            TP=TP+1;
        else FP=FP+1;
    end
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==0
        if ad(i,label)==0
            TN=TN+1;
        else FN=FN+1;
        end
    end
end

TPR = (TP/P)*100;
FPR = (FP/N)*100;
OSR = ((TP+TN)/(TP+FP+TN+FN))*100;
Precision = (TP/(TP+FP))*100;
F_score = 2*(((Precision/100)*(TPR/100))/((Precision/100)+(TPR/100)));

%disp (P); disp (N); disp (TP); disp (TN); disp (FP); disp (FN); disp (TPR); disp (FPR);

% write the results into (ds_all_results) matrix

ds_all_metrics_results = [ds_all_metrics_results; P N TP TN FP FN TPR FPR OSR Precision F_score];

end

% apply Dempster-Shafer on two metrics

for i=1:size(ad,1)
    
    % This for loop is to apply the Dempster-Shafer theory on two metrics
    % i.e. try all the cominations of two metrics fusion
    
    % for deauth data comment all the combinations related to TTL
    
% RSSI with the other metrics
    
    % 1- dempster-shafer for RSSI and Injection rate
    % dempster shafer beliefs are written into coloumns 71,72,73 (attack, normal, uncertainty)
    % the frame label is written into coloumn 74
    
    [ad(i,71),ad(i,72),ad(i,73),ad(i,74)] = ds(ad(i,11),ad(i,21),ad(i,31),ad(i,12),ad(i,22),ad(i,32));
    
    %  dempster-shafer for RSSI and TTL
    % dempster shafer beliefs are written into coloumns 76,77,78 (attack, normal, uncertainty)
    % the frame label is written into coloumn 79
    
    %[ad(i,76),ad(i,77),ad(i,78),ad(i,79)] = ds(ad(i,11),ad(i,21),ad(i,31),ad(i,13),ad(i,23),ad(i,33));

    % 2- dempster-shafer for RSSI and NAV
    % dempster shafer beliefs are written into coloumns 81,82,83 (attack, normal, uncertainty)
    % the frame label is written into coloumn 84
    
    [ad(i,81),ad(i,82),ad(i,83),ad(i,84)] = ds(ad(i,11),ad(i,21),ad(i,31),ad(i,14),ad(i,24),ad(i,34));    
    
    % 3- dempster-shafer for RSSI and sequence number
    % dempster shafer beliefs are written into coloumns 86,87,88 (attack, normal, uncertainty)
    % the frame label is written into coloumn 89
    
    [ad(i,86),ad(i,87),ad(i,88),ad(i,89)] = ds(ad(i,11),ad(i,21),ad(i,31),ad(i,15),ad(i,25),ad(i,35));    

    % 4- dempster-shafer for RSSI and Inter-arrival time
    % dempster shafer beliefs are written into coloumns 91,92,93 (attack, normal, uncertainty)
    % the frame label is written into coloumn 94
    
    [ad(i,91),ad(i,92),ad(i,93),ad(i,94)] = ds(ad(i,11),ad(i,21),ad(i,31),ad(i,16),ad(i,26),ad(i,36));    
    
% Injection rate with the other metrics
    
    % - dempster-shafer for Injection rate and TTL
    % dempster shafer beliefs are written into coloumns 96,97,98 (attack, normal, uncertainty)
    % the frame label is written into coloumn 99
    
    %[ad(i,96),ad(i,97),ad(i,98),ad(i,99)] = ds(ad(i,12),ad(i,22),ad(i,32),ad(i,13),ad(i,23),ad(i,33));    
    
    % 5- dempster-shafer for Injection rate and NAV
    % dempster shafer beliefs are written into coloumns 101,102,103 (attack, normal, uncertainty)
    % the frame label is written into coloumn 104
    
    [ad(i,101),ad(i,102),ad(i,103),ad(i,104)] = ds(ad(i,12),ad(i,22),ad(i,32),ad(i,14),ad(i,24),ad(i,34));    
    
    % 6- dempster-shafer for Injection rate and sequence number
    % dempster shafer beliefs are written into coloumns 106,107,108 (attack, normal, uncertainty)
    % the frame label is written into coloumn 109
    
    [ad(i,106),ad(i,107),ad(i,108),ad(i,109)] = ds(ad(i,12),ad(i,22),ad(i,32),ad(i,15),ad(i,25),ad(i,35));
    
    % 7- dempster-shafer for Injection rate and Inter-arrival time
    % dempster shafer beliefs are written into coloumns 111,112,113 (attack, normal, uncertainty)
    % the frame label is written into coloumn 114
    
    [ad(i,111),ad(i,112),ad(i,113),ad(i,114)] = ds(ad(i,12),ad(i,22),ad(i,32),ad(i,16),ad(i,26),ad(i,36));    

% TTL with the other metrics
    
    % - dempster-shafer for TTL and NAV
    % dempster shafer beliefs are written into coloumns 116,117,118 (attack, normal, uncertainty)
    % the frame label is written into coloumn 119
    
    %[ad(i,116),ad(i,117),ad(i,118),ad(i,119)] = ds(ad(i,13),ad(i,23),ad(i,33),ad(i,14),ad(i,24),ad(i,34));    
    
    % - dempster-shafer for TTL and sequence number
    % dempster shafer beliefs are written into coloumns 121,122,123 (attack, normal, uncertainty)
    % the frame label is written into coloumn 124
    
    %[ad(i,121),ad(i,122),ad(i,123),ad(i,124)] = ds(ad(i,13),ad(i,23),ad(i,33),ad(i,15),ad(i,25),ad(i,35)); 
    
    % - dempster-shafer for TTL and Inter-arrival time
    % dempster shafer beliefs are written into coloumns 126,127,128 (attack, normal, uncertainty)
    % the frame label is written into coloumn 129
    
    %[ad(i,126),ad(i,127),ad(i,128),ad(i,129)] = ds(ad(i,13),ad(i,23),ad(i,33),ad(i,16),ad(i,26),ad(i,36));    
    
% NAV with the other metrics
    
    % 8- dempster-shafer for NAV and sequence number
    % dempster shafer beliefs are written into coloumns 131,132,133 (attack, normal, uncertainty)
    % the frame label is written into coloumn 134
    
    [ad(i,131),ad(i,132),ad(i,133),ad(i,134)] = ds(ad(i,14),ad(i,24),ad(i,34),ad(i,15),ad(i,25),ad(i,35));    

    % 9- dempster-shafer for NAV and Inter-arrival time
    % dempster shafer beliefs are written into coloumns 136,137,138 (attack, normal, uncertainty)
    % the frame label is written into coloumn 139
    
    [ad(i,136),ad(i,137),ad(i,138),ad(i,139)] = ds(ad(i,14),ad(i,24),ad(i,34),ad(i,16),ad(i,26),ad(i,36));    
    
% sequence number with the other metrics
    
    % 10- dempster-shafer for sequence number and Inter-arrival time
    % dempster shafer beliefs are written into coloumns 141,142,143 (attack, normal, uncertainty)
    % the frame label is written into coloumn 144
    
    [ad(i,141),ad(i,142),ad(i,143),ad(i,144)] = ds(ad(i,15),ad(i,25),ad(i,35),ad(i,16),ad(i,26),ad(i,36));    
      
end

% This is to find the results of Dempster-Shafer on all metrics, 
% i.e. calculate the TPR and FPR of each fusion step.

% calculate P, N, TP, TN, FP, FN, TPR, FPR

label=7;     % ground truth

ds_two_metrics_results = [];  % to write the results into

for ds_l = [74 84 89 94 104 109 114 134 139 144]   %dempster-shafer label 
                %79, 99, 119, 124, 129 lables were deleted for deauth data
    
    P=0; N=0; TP=0; TN=0; FP=0; FN=0;

for i=1:size(ad,1)
    if ad(i,label)==1
        P=P+1;
    else N=N+1;
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==1
        if ad(i,label)==1
            TP=TP+1;
        else FP=FP+1;
    end
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==0
        if ad(i,label)==0
            TN=TN+1;
        else FN=FN+1;
        end
    end
end

TPR = (TP/P)*100;
FPR = (FP/N)*100;
OSR = ((TP+TN)/(TP+FP+TN+FN))*100;
Precision = (TP/(TP+FP))*100;
F_score = 2*(((Precision/100)*(TPR/100))/((Precision/100)+(TPR/100)));

%disp (P); disp (N); disp (TP); disp (TN); disp (FP); disp (FN); disp (TPR); disp (FPR);

% write the results into (ds_all_results) matrix

ds_two_metrics_results = [ds_two_metrics_results; P N TP TN FP FN TPR FPR OSR Precision F_score];

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% apply Dempster-Shafer on three metrics, for deauth data: comment all TTL
% calculation, and delete all TTL labels

for i=1:size(ad,1)
    
    % This for loop is to apply the Dempster-Shafer theory on three metrics
    % i.e. try all the cominations of three metrics fusion

% RSSI, Rate

    % - dempster-shafer for (RSSI, Rate, TTL)
    % dempster shafer beliefs are written into coloumns 151,152,153 (attack, normal, uncertainty)
    % the frame label is written into coloumn 154
    
    %[ad(i,151),ad(i,152),ad(i,153),ad(i,154)] = ds(ad(i,71),ad(i,72),ad(i,73),ad(i,13),ad(i,23),ad(i,33));
    
    % 1- dempster-shafer for (RSSI, Rate, NAV)
    % dempster shafer beliefs are written into coloumns 156,157,158 (attack, normal, uncertainty)
    % the frame label is written into coloumn 159
    
    [ad(i,156),ad(i,157),ad(i,158),ad(i,159)] = ds(ad(i,71),ad(i,72),ad(i,73),ad(i,14),ad(i,24),ad(i,34));
    
    % 2- dempster-shafer for (RSSI, Rate, Seq)
    % dempster shafer beliefs are written into coloumns 161,162,163 (attack, normal, uncertainty)
    % the frame label is written into coloumn 164
    
    [ad(i,161),ad(i,162),ad(i,163),ad(i,164)] = ds(ad(i,71),ad(i,72),ad(i,73),ad(i,15),ad(i,25),ad(i,35));

    % 3- dempster-shafer for (RSSI, Rate, IAT)
    % dempster shafer beliefs are written into coloumns 166,167,168 (attack, normal, uncertainty)
    % the frame label is written into coloumn 169
    
    [ad(i,166),ad(i,167),ad(i,168),ad(i,169)] = ds(ad(i,71),ad(i,72),ad(i,73),ad(i,16),ad(i,26),ad(i,36));    

% RSSI, TTL    

    % - dempster-shafer for (RSSI, TTL, NAV)
    % dempster shafer beliefs are written into coloumns 171,172,173 (attack, normal, uncertainty)
    % the frame label is written into coloumn 174
    
   % [ad(i,171),ad(i,172),ad(i,173),ad(i,174)] = ds(ad(i,76),ad(i,77),ad(i,78),ad(i,14),ad(i,24),ad(i,34));

    % - dempster-shafer for (RSSI, TTL, Seq)
    % dempster shafer beliefs are written into coloumns 176,177,178 (attack, normal, uncertainty)
    % the frame label is written into coloumn 179
    
    %[ad(i,176),ad(i,177),ad(i,178),ad(i,179)] = ds(ad(i,76),ad(i,77),ad(i,78),ad(i,15),ad(i,25),ad(i,35));

    % - dempster-shafer for (RSSI, TTL, IAT)
    % dempster shafer beliefs are written into coloumns 181,182,183 (attack, normal, uncertainty)
    % the frame label is written into coloumn 184
    
    %[ad(i,181),ad(i,182),ad(i,183),ad(i,184)] = ds(ad(i,76),ad(i,77),ad(i,78),ad(i,16),ad(i,26),ad(i,36));

% RSSI, NAV

    % 4- dempster-shafer for (RSSI, NAV, Seq)
    % dempster shafer beliefs are written into coloumns 186,187,188 (attack, normal, uncertainty)
    % the frame label is written into coloumn 189
    
    [ad(i,186),ad(i,187),ad(i,188),ad(i,189)] = ds(ad(i,81),ad(i,82),ad(i,83),ad(i,15),ad(i,25),ad(i,35));

    % 5- dempster-shafer for (RSSI, NAV, IAT)
    % dempster shafer beliefs are written into coloumns 191,192,193 (attack, normal, uncertainty)
    % the frame label is written into coloumn 194
    
    [ad(i,191),ad(i,192),ad(i,193),ad(i,194)] = ds(ad(i,81),ad(i,82),ad(i,83),ad(i,16),ad(i,26),ad(i,36));

    % 6- dempster-shafer for (RSSI, Seq, IAT)
    % dempster shafer beliefs are written into coloumns 196,197,198 (attack, normal, uncertainty)
    % the frame label is written into coloumn 199
    
    [ad(i,196),ad(i,197),ad(i,198),ad(i,199)] = ds(ad(i,86),ad(i,87),ad(i,88),ad(i,16),ad(i,26),ad(i,36));    
    
% Rate, TTL

    % - dempster-shafer for (Rate, TTL, NAV)
    % dempster shafer beliefs are written into coloumns 201,202,203 (attack, normal, uncertainty)
    % the frame label is written into coloumn 204
    
    %[ad(i,201),ad(i,202),ad(i,203),ad(i,204)] = ds(ad(i,96),ad(i,97),ad(i,98),ad(i,14),ad(i,24),ad(i,34));
    
    % - dempster-shafer for (Rate, TTL, Seq)
    % dempster shafer beliefs are written into coloumns 206,207,208 (attack, normal, uncertainty)
    % the frame label is written into coloumn 209
    
    %[ad(i,206),ad(i,207),ad(i,208),ad(i,209)] = ds(ad(i,96),ad(i,97),ad(i,98),ad(i,15),ad(i,25),ad(i,35));    
    
    % - dempster-shafer for (Rate, TTL, IAT)
    % dempster shafer beliefs are written into coloumns 211,212,213 (attack, normal, uncertainty)
    % the frame label is written into coloumn 214
    
    %[ad(i,211),ad(i,212),ad(i,213),ad(i,214)] = ds(ad(i,96),ad(i,97),ad(i,98),ad(i,16),ad(i,26),ad(i,36));    

% Rate, NAV

    % 7- dempster-shafer for (Rate, NAV, Seq)
    % dempster shafer beliefs are written into coloumns 216,217,218 (attack, normal, uncertainty)
    % the frame label is written into coloumn 219
    
    [ad(i,216),ad(i,217),ad(i,218),ad(i,219)] = ds(ad(i,101),ad(i,102),ad(i,103),ad(i,15),ad(i,25),ad(i,35));    
    
    % 8- dempster-shafer for (Rate, NAV, IAT)
    % dempster shafer beliefs are written into coloumns 221,222,223 (attack, normal, uncertainty)
    % the frame label is written into coloumn 224
    
    [ad(i,221),ad(i,222),ad(i,223),ad(i,224)] = ds(ad(i,101),ad(i,102),ad(i,103),ad(i,16),ad(i,26),ad(i,36));   

% RAte, Seq

    % 9- dempster-shafer for (Rate, Seq, IAT)
    % dempster shafer beliefs are written into coloumns 226,227,228 (attack, normal, uncertainty)
    % the frame label is written into coloumn 229
    
    [ad(i,226),ad(i,227),ad(i,228),ad(i,229)] = ds(ad(i,106),ad(i,107),ad(i,108),ad(i,16),ad(i,26),ad(i,36));    
    
% TTL, NAV

    % - dempster-shafer for (TTL, NAV, Seq)
    % dempster shafer beliefs are written into coloumns 231,232,233 (attack, normal, uncertainty)
    % the frame label is written into coloumn 234
    
    %[ad(i,231),ad(i,232),ad(i,233),ad(i,234)] = ds(ad(i,116),ad(i,117),ad(i,118),ad(i,15),ad(i,25),ad(i,35));

% TTL, Seq

    % - dempster-shafer for (TTL, Seq, IAT)
    % dempster shafer beliefs are written into coloumns 236,237,238 (attack, normal, uncertainty)
    % the frame label is written into coloumn 239
    
    %[ad(i,236),ad(i,237),ad(i,238),ad(i,239)] = ds(ad(i,121),ad(i,122),ad(i,123),ad(i,16),ad(i,26),ad(i,36));
    
% NAV, Seq

    % 10- dempster-shafer for (NAV, Seq, IAT)
    % dempster shafer beliefs are written into coloumns 241,242,243 (attack, normal, uncertainty)
    % the frame label is written into coloumn 244
    
    [ad(i,241),ad(i,242),ad(i,243),ad(i,244)] = ds(ad(i,131),ad(i,132),ad(i,133),ad(i,16),ad(i,26),ad(i,36));    
       
end

% This is to find the results of Dempster-Shafer on 3 metrics
% i.e. calculate the TPR, FPR and OSR of each fusion step.

% calculate P, N, TP, TN, FP, FN, TPR, FPR

label=7;     % ground truth

ds_three_metrics_results = [];  % to write the results into

for ds_l = [159 164 169 189 194 199 219 224 229 244]   
           % ds_l : dempster-shafer labels
       % those TTL labels were deleted 154 174 179 184 204 209 214 234 239 
    
    P=0; N=0; TP=0; TN=0; FP=0; FN=0;

for i=1:size(ad,1)
    if ad(i,label)==1
        P=P+1;
    else N=N+1;
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==1
        if ad(i,label)==1
            TP=TP+1;
        else FP=FP+1;
    end
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==0
        if ad(i,label)==0
            TN=TN+1;
        else FN=FN+1;
        end
    end
end

TPR = (TP/P)*100;
FPR = (FP/N)*100;
OSR = ((TP+TN)/(TP+FP+TN+FN))*100;
Precision = (TP/(TP+FP))*100;
F_score = 2*(((Precision/100)*(TPR/100))/((Precision/100)+(TPR/100)));

%disp (P); disp (N); disp (TP); disp (TN); disp (FP); disp (FN); disp (TPR); disp (FPR);

% write the results into (ds_all_results) matrix

ds_three_metrics_results = [ds_three_metrics_results; P N TP TN FP FN TPR FPR OSR Precision F_score];

end

% round the ad_three_metrics results into 2 decimal digits

for i=1:size(ds_three_metrics_results,1)
    for j=1:size(ds_three_metrics_results,2)
        ds_three_metrics_results(i,j) = round(ds_three_metrics_results(i,j),2);        
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% apply Dempster-Shafer on four metrics

for i=1:size(ad,1)
    
    % This for loop is to apply the Dempster-Shafer theory on four metrics
    % i.e. try all the cominations of four metrics fusion

% RSSI, Rate, TTl

    % - dempster-shafer for (RSSI, Rate, TTL, NAV)
    % dempster shafer beliefs are written into coloumns 251,252,253 (attack, normal, uncertainty)
    % the frame label is written into coloumn 254
    
%    [ad(i,251),ad(i,252),ad(i,253),ad(i,254)] = ds(ad(i,151),ad(i,152),ad(i,153),ad(i,14),ad(i,24),ad(i,34));
 
    % - dempster-shafer for (RSSI, Rate, TTL, Seq)   
%    [ad(i,256),ad(i,257),ad(i,258),ad(i,259)] = ds(ad(i,151),ad(i,152),ad(i,153),ad(i,15),ad(i,25),ad(i,35));
    
    % - dempster-shafer for (RSSI, Rate, TTL, IAT)   
%    [ad(i,261),ad(i,262),ad(i,263),ad(i,264)] = ds(ad(i,151),ad(i,152),ad(i,153),ad(i,16),ad(i,26),ad(i,36));   

% RSSI, Rate, NAV    
    
    % 1- dempster-shafer for (RSSI, Rate, NAV, Seq)   
    [ad(i,266),ad(i,267),ad(i,268),ad(i,269)] = ds(ad(i,156),ad(i,157),ad(i,158),ad(i,15),ad(i,25),ad(i,35));    
    
    % 2- dempster-shafer for (RSSI, Rate, NAV, IAT)   
    [ad(i,271),ad(i,272),ad(i,273),ad(i,274)] = ds(ad(i,156),ad(i,157),ad(i,158),ad(i,16),ad(i,26),ad(i,36));    
    
% RSSI, Rate, Seq

    % 3- dempster-shafer for (RSSI, Rate, Seq, IAT)   
    [ad(i,276),ad(i,277),ad(i,278),ad(i,279)] = ds(ad(i,161),ad(i,162),ad(i,163),ad(i,16),ad(i,26),ad(i,36));
    
% RSSI, TTL, NAV

    % - dempster-shafer for (RSSI, TTL, NAV, Seq)   
%    [ad(i,281),ad(i,282),ad(i,283),ad(i,284)] = ds(ad(i,171),ad(i,172),ad(i,173),ad(i,15),ad(i,25),ad(i,35));    
    
    % - dempster-shafer for (RSSI, TTL, NAV, IAT)   
%    [ad(i,286),ad(i,287),ad(i,288),ad(i,289)] = ds(ad(i,171),ad(i,172),ad(i,173),ad(i,16),ad(i,26),ad(i,36));  
    
% RSSI, TTL, Seq

    % - dempster-shafer for (RSSI, TTL, Seq, IAT)   
%    [ad(i,291),ad(i,292),ad(i,293),ad(i,294)] = ds(ad(i,176),ad(i,177),ad(i,178),ad(i,16),ad(i,26),ad(i,36));    
    
% RSSI, NAV, Seq

    % 4- dempster-shafer for (RSSI, NAV, Seq, IAT)   
    [ad(i,296),ad(i,297),ad(i,298),ad(i,299)] = ds(ad(i,186),ad(i,187),ad(i,188),ad(i,16),ad(i,26),ad(i,36));    

% Rate, TTL, NAV

    % - dempster-shafer for (Rate, TTL, NAV, Seq)   
%    [ad(i,301),ad(i,302),ad(i,303),ad(i,304)] = ds(ad(i,201),ad(i,202),ad(i,203),ad(i,15),ad(i,25),ad(i,35));    
    
    % - dempster-shafer for (Rate, TTL, NAV, IAT)   
%    [ad(i,306),ad(i,307),ad(i,308),ad(i,309)] = ds(ad(i,201),ad(i,202),ad(i,203),ad(i,16),ad(i,26),ad(i,36));  
    
%

    % - dempster-shafer for (Rate, TTL, Seq, IAT)   
%    [ad(i,311),ad(i,312),ad(i,313),ad(i,314)] = ds(ad(i,206),ad(i,207),ad(i,208),ad(i,16),ad(i,26),ad(i,36));
    
    % 5- dempster-shafer for (Rate, NAV, Seq, IAT)   
    [ad(i,316),ad(i,317),ad(i,318),ad(i,319)] = ds(ad(i,216),ad(i,217),ad(i,218),ad(i,16),ad(i,26),ad(i,36));    
    
    % - dempster-shafer for (TTL, NAV, Seq, IAT)   
%    [ad(i,321),ad(i,322),ad(i,323),ad(i,324)] = ds(ad(i,231),ad(i,232),ad(i,233),ad(i,16),ad(i,26),ad(i,36));    
   
end

% This is to find the results of Dempster-Shafer on 4 metrics
% i.e. calculate the TPR, FPR and OSR of each fusion step.

% calculate P, N, TP, TN, FP, FN, TPR, FPR

label=7;     % ground truth

ds_four_metrics_results = [];  % to write the results into

for ds_l = [269 274 279 299 319]   
           % ds_l : dempster-shafer labels
        % 254 259 264 284 289 294 304 309 314 324 deleted labels (TTL ones)
    
    P=0; N=0; TP=0; TN=0; FP=0; FN=0;

for i=1:size(ad,1)
    if ad(i,label)==1
        P=P+1;
    else N=N+1;
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==1
        if ad(i,label)==1
            TP=TP+1;
        else FP=FP+1;
    end
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==0
        if ad(i,label)==0
            TN=TN+1;
        else FN=FN+1;
        end
    end
end

TPR = (TP/P)*100;
FPR = (FP/N)*100;
OSR = ((TP+TN)/(TP+FP+TN+FN))*100;
Precision = (TP/(TP+FP))*100;
F_score = 2*(((Precision/100)*(TPR/100))/((Precision/100)+(TPR/100)));

%disp (P); disp (N); disp (TP); disp (TN); disp (FP); disp (FN); disp (TPR); disp (FPR);

% write the results into (ds_all_results) matrix

ds_four_metrics_results = [ds_four_metrics_results; P N TP TN FP FN TPR FPR OSR Precision F_score];

end

% round the ad_three_metrics results into 2 decimal digits

for i=1:size(ds_four_metrics_results,1)
    for j=1:size(ds_four_metrics_results,2)
        ds_four_metrics_results(i,j) = round(ds_four_metrics_results(i,j),2);        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% apply Dempster-Shafer on five metrics

for i=1:size(ad,1)
    
    % This for loop is to apply the Dempster-Shafer theory on five metrics
    % i.e. try all the cominations of four metrics fusion

% RSSI, Rate, TTl, NAV, Seq

    % - dempster-shafer for (RSSI, Rate, TTl, NAV, Seq)
    % dempster shafer beliefs are written into coloumns 331,332,333 (attack, normal, uncertainty)
    % the frame label is written into coloumn 334
    
%    [ad(i,331),ad(i,332),ad(i,333),ad(i,334)] = ds(ad(i,251),ad(i,252),ad(i,153),ad(i,15),ad(i,25),ad(i,35));
 
    % - dempster-shafer for (RSSI, Rate, TTl, NAV, IAT)    
%    [ad(i,336),ad(i,337),ad(i,338),ad(i,339)] = ds(ad(i,261),ad(i,262),ad(i,163),ad(i,16),ad(i,26),ad(i,36));

    % - dempster-shafer for (RSSI, Rate, TTl, Seq, IAT)    
%    [ad(i,341),ad(i,342),ad(i,343),ad(i,344)] = ds(ad(i,256),ad(i,257),ad(i,158),ad(i,16),ad(i,26),ad(i,36));    
    
    % 1- dempster-shafer for (RSSI, Rate, NAV, Seq, IAT)    
    [ad(i,346),ad(i,347),ad(i,348),ad(i,349)] = ds(ad(i,266),ad(i,267),ad(i,168),ad(i,16),ad(i,26),ad(i,36));
    
    % - dempster-shafer for (RSSI, TTL, NAV, Seq, IAT)    
%    [ad(i,351),ad(i,352),ad(i,353),ad(i,354)] = ds(ad(i,281),ad(i,282),ad(i,183),ad(i,16),ad(i,26),ad(i,36));    
    
    % - dempster-shafer for (Rate, TTL, NAV, Seq, IAT)    
%    [ad(i,356),ad(i,357),ad(i,358),ad(i,359)] = ds(ad(i,301),ad(i,302),ad(i,303),ad(i,16),ad(i,26),ad(i,36));   
    
end

% This is to find the results of Dempster-Shafer on 5 metrics
% i.e. calculate the TPR, FPR and OSR of each fusion step.

% calculate P, N, TP, TN, FP, FN, TPR, FPR

label=7;     % ground truth

ds_five_metrics_results = [];  % to write the results into

for ds_l = [349]   
           % ds_l : dempster-shafer labels
           %334 339 344 354 359 deleted labels (no TTL here) 
    
    P=0; N=0; TP=0; TN=0; FP=0; FN=0;

for i=1:size(ad,1)
    if ad(i,label)==1
        P=P+1;
    else N=N+1;
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==1
        if ad(i,label)==1
            TP=TP+1;
        else FP=FP+1;
    end
    end
end

for i=1:size(ad,1)
    if ad(i,ds_l)==0
        if ad(i,label)==0
            TN=TN+1;
        else FN=FN+1;
        end
    end
end

TPR = (TP/P)*100;
FPR = (FP/N)*100;
OSR = ((TP+TN)/(TP+FP+TN+FN))*100;
Precision = (TP/(TP+FP))*100;
F_score = 2*(((Precision/100)*(TPR/100))/((Precision/100)+(TPR/100)));

%disp (P); disp (N); disp (TP); disp (TN); disp (FP); disp (FN); disp (TPR); disp (FPR);

% write the results into (ds_all_results) matrix

ds_five_metrics_results = [ds_five_metrics_results; P N TP TN FP FN TPR FPR OSR Precision F_score];

end

% round the ad_three_metrics results into 2 decimal digits

for i=1:size(ds_five_metrics_results,1)
    for j=1:size(ds_five_metrics_results,2)
        ds_five_metrics_results(i,j) = round(ds_five_metrics_results(i,j),2);        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% this is to plot the combination results

ds_all_combinations_results = [ds_two_metrics_results(:,7:9); ... 
    ds_three_metrics_results(:,7:9); ds_four_metrics_results(:,7:9); ...
    ds_five_metrics_results(1,7:9)];

figure('rend','painters','pos',[10 10 1000 510]);
combinations_results = bar(ds_all_combinations_results);
set(gca,'XTick',[1:26],'XTickLabel',{'RSSI, Rate','RSSI, NAV','RSSI, Seq', ...
    'RSSI, IAT', 'Rate, NAV', 'Rate, Seq', 'Rate, IAT', ...
    'NAV, Seq', 'NAV, IAT', 'Seq, IAT', ...
    'RSSI, Rate, NAV', 'RSSI, Rate, Seq', 'RSSI, Rate, IAT', ...
    'RSSI, NAV, Seq', ...
    'RSSI, NAV, IAT', 'RSSI, Seq, IAT', ...
    'Rate, NAV, Seq', 'Rate, NAV, IAT', 'Rate, Seq, IAT', ...
    'NAV, Seq, IAT', ...
    'RSSI, Rate, NAV, Seq', 'RSSI, Rate, NAV, IAT', 'RSSI, Rate, Seq, IAT', ...
    'RSSI, NAV, Seq, IAT', ...
    'Rate, NAV, Seq, IAT', ...
    'RSSI, Rate, NAV, Seq, IAT'});
set(gca,'XTickLabelRotation',90);
combinations_results(1).FaceColor = 'g';
combinations_results(2).FaceColor = 'r';
combinations_results(3).FaceColor = 'b';
ylabel('Percentage (%)');
legend('TPR', 'FPR', 'OSR');
set(legend,'Orientation','horizontal','Location','northoutside');
% this is to clear the white spapce when you save to pdf file
fig = gcf;
fig.PaperPositionMode = 'auto';
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
